package graphvent

import (
  "testing"
  "time"
  "fmt"
  "encoding/json"
)

func TestGQLThread(t * testing.T) {
  ctx := logTestContext(t, []string{})
  gql_thread, err := NewGQLThread(ctx, ":8080", []Lockable{})
  fatalErr(t, err)

  test_thread_1, err := NewSimpleThread(ctx, "Test thread 1", []Lockable{}, BaseThreadActions, BaseThreadHandlers)
  fatalErr(t, err)

  test_thread_2, err := NewSimpleThread(ctx, "Test thread 2", []Lockable{}, BaseThreadActions, BaseThreadHandlers)
  fatalErr(t, err)

  i1 := NewGQLThreadInfo(true, "start", "restore")
  err = LinkThreads(ctx, gql_thread, test_thread_1, &i1)
  fatalErr(t, err)

  i2 := NewGQLThreadInfo(false, "start", "restore")
  err = LinkThreads(ctx, gql_thread, test_thread_2, &i2)
  fatalErr(t, err)

  go func(thread Thread){
    time.Sleep(10*time.Millisecond)
    SendUpdate(ctx, thread, CancelSignal(nil))
  }(gql_thread)

  err = RunThread(ctx, gql_thread, "start")
  fatalErr(t, err)
}

func TestGQLDBLoad(t * testing.T) {
  ctx := logTestContext(t, []string{"thread"})
  l1, err := NewSimpleLockable(ctx, "Test Lockable 1", []Lockable{})
  fatalErr(t, err)

  t1, err := NewSimpleThread(ctx, "Test Thread 1", []Lockable{}, BaseThreadActions, BaseThreadHandlers)
  fatalErr(t, err)
  update_channel := t1.UpdateChannel(10)

  gql, err := NewGQLThread(ctx, ":8080", []Lockable{l1})
  fatalErr(t, err)

  info := NewGQLThreadInfo(true, "start", "restore")
  err = LinkThreads(ctx, gql, t1, &info)
  fatalErr(t, err)

  SendUpdate(ctx, gql, CancelSignal(nil))
  err = RunThread(ctx, gql, "start")
  fatalErr(t, err)

  (*GraphTester)(t).WaitForValue(ctx, update_channel, "thread_done", t1, 100*time.Millisecond, "Dicn't received update_done on t1 from t1")

  err = UseStates(ctx, []GraphNode{gql, t1}, func(states NodeStateMap) error {
    ser1, err := json.MarshalIndent(states[gql.ID()], "", "  ")
    ser2, err := json.MarshalIndent(states[t1.ID()], "", "  ")
    fmt.Printf("\n%s\n\n", ser1)
    fmt.Printf("\n%s\n\n", ser2)
    return err
  })

  gql_loaded, err := LoadNode(ctx, gql.ID())
  fatalErr(t, err)
  var t1_loaded *BaseThread = nil

  err = UseStates(ctx, []GraphNode{gql_loaded}, func(states NodeStateMap) error {
    ser, err := json.MarshalIndent(states[gql_loaded.ID()], "", "  ")
    fmt.Printf("\n%s\n\n", ser)
    child := states[gql_loaded.ID()].(ThreadState).Children()[0]
    t1_loaded = child.(*BaseThread)
    update_channel = t1_loaded.UpdateChannel(10)
    err = UseMoreStates(ctx, []GraphNode{child}, states, func(states NodeStateMap) error {
      ser, err := json.MarshalIndent(states[child.ID()], "", "  ")
      fmt.Printf("\n%s\n\n", ser)
      return err
    })
    return err
  })

  SendUpdate(ctx, gql_loaded, CancelSignal(nil))
  err = RunThread(ctx, gql_loaded.(Thread), "restore")
  fatalErr(t, err)
  (*GraphTester)(t).WaitForValue(ctx, update_channel, "thread_done", t1_loaded, 100*time.Millisecond, "Dicn't received update_done on t1_loaded from t1_loaded")

}
